/**
 * Copyright (c) 2000-2013 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of the
 * GNU Lesser General Public License as published by the Free Software Foundation; either version
 * 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 */

package it.tref.liferay.mapit.map.portlet;

import it.tref.liferay.mapit.map.model.PluginPortletPreferences;
import it.tref.liferay.mapit.map.service.PluginPortletPreferencesLocalServiceUtil;

import java.util.Enumeration;
import java.util.List;

import javax.portlet.PortletPreferences;

import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portal.kernel.lar.DataLevel;
import com.liferay.portal.kernel.lar.DefaultConfigurationPortletDataHandler;
import com.liferay.portal.kernel.lar.PortletDataContext;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.PortletConstants;
import com.liferay.portal.util.PortletKeys;


public class MapItPortletDataHandler extends DefaultConfigurationPortletDataHandler {

  private final String MAP_IT_PLUGIN_PREFIX = "MAP_IT_PLUGIN_";

  public MapItPortletDataHandler() {
    setDataLevel(DataLevel.PORTLET_INSTANCE);
    setPublishToLiveByDefault(true);
  }

  @Override
  protected PortletPreferences doProcessExportPortletPreferences(PortletDataContext portletDataContext,
      String portletId, PortletPreferences portletPreferences) throws Exception {

    long plid = portletDataContext.getPlid();

    int ownerType = PortletKeys.PREFS_OWNER_TYPE_LAYOUT;

    if (PortletConstants.hasUserId(portletId)) {
      ownerType = PortletKeys.PREFS_OWNER_TYPE_USER;
    }

    List<PluginPortletPreferences> pluginsPortletPreferences =
        PluginPortletPreferencesLocalServiceUtil.getPortletPreferences(ownerType, plid, portletId, QueryUtil.ALL_POS,
            QueryUtil.ALL_POS);

    for (PluginPortletPreferences pluginPortletPreferences : pluginsPortletPreferences) {

      portletPreferences.setValue(MAP_IT_PLUGIN_PREFIX + pluginPortletPreferences.getPlugin(),
          pluginPortletPreferences.getPreferences());
    }

    return portletPreferences;
  }

  @Override
  protected PortletPreferences doProcessImportPortletPreferences(PortletDataContext portletDataContext,
      String portletId, PortletPreferences portletPreferences) throws Exception {

    long plid = portletDataContext.getPlid();

    long ownerId = PortletKeys.PREFS_OWNER_ID_DEFAULT;
    int ownerType = PortletKeys.PREFS_OWNER_TYPE_LAYOUT;

    if (PortletConstants.hasUserId(portletId)) {
      ownerId = PortletConstants.getUserId(portletId);
      ownerType = PortletKeys.PREFS_OWNER_TYPE_USER;
    }

    Enumeration<String> enumeration = portletPreferences.getNames();

    while (enumeration.hasMoreElements()) {

      String key = enumeration.nextElement();

      if (key.startsWith(MAP_IT_PLUGIN_PREFIX)) {

        String value = portletPreferences.getValue(key, null);

        if (Validator.isNotNull(value)) {

          String plugin = key.substring(MAP_IT_PLUGIN_PREFIX.length());

          PluginPortletPreferencesLocalServiceUtil
              .updatePreferences(ownerId, ownerType, plid, portletId, plugin, value);
        }
      }
    }

    return portletPreferences;
  }
}
