package it.tref.liferay.mapit.map.util;

import it.tref.liferay.mapit.map.model.Plugin;
import it.tref.liferay.mapit.map.service.PluginLocalServiceUtil;
import it.tref.liferay.mapit.plugin.MapItPlugin;
import it.tref.liferay.mapit.plugin.event.MapItPluginEvent;
import it.tref.liferay.mapit.plugin.event.MapItPluginEvent.MapItPluginEventType;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import com.liferay.portal.kernel.messaging.MessageBusUtil;
import com.liferay.portal.kernel.util.ClassLoaderPool;
import com.liferay.portal.kernel.util.ClassResolverUtil;
import com.liferay.portal.kernel.util.ReflectionUtil;
import com.liferay.portal.kernel.util.Validator;

public class MapItPluginEventUtil {

  public static String[] getDestinationNames(String[] pluginIds, MapItPluginEventType mapItPluginEventType)
      throws Exception {

    List<String> destinationNames = new ArrayList<String>();

    for (String pluginId : pluginIds) {

      Plugin plugin = PluginLocalServiceUtil.getPlugin(pluginId);

      if (plugin != null && plugin.isActive()) {

        MapItPlugin mapItPlugin = MapItPluginUtil.getMapItPlugin(plugin.getContext());
        Set<Class<? extends MapItPluginEvent>> mapItPluginEvents = mapItPlugin.getMapItPluginEvents();

        if (mapItPluginEvents != null) {
          for (Class<? extends MapItPluginEvent> event : mapItPluginEvents) {

            ClassLoader contextClassLoader = ClassLoaderPool.getClassLoader(plugin.getContext());
            Class<?> clazz = ClassResolverUtil.resolve(event.getName(), contextClassLoader);
            Object obj = clazz.getConstructor(new Class[0]).newInstance(new Object[0]);

            Object eventType = ReflectionUtil.getDeclaredMethod(clazz, "getEventType").invoke(obj);
            String destinationName = (String) ReflectionUtil.getDeclaredMethod(clazz, "getDestinationName").invoke(obj);

            if (eventType != null && Validator.isNotNull(destinationName)) {
              if (MapItPluginEventType.parse(eventType.toString()).equals(mapItPluginEventType)) {
                if (MessageBusUtil.hasMessageListener(destinationName)) {
                  destinationNames.add(destinationName);
                }
              }
            }
          }
        }
      }
    }

    return destinationNames.toArray(new String[destinationNames.size()]);
  }

}
